@@ -51,7 +51,7 @@ setMethodS3("getChecksumObjects", "GenericDataFileSet", function(this, ..., verb
 
   verbose && enter(verbose, "Retrieving checksum object for identifying this ", class(this)[1]);
 
-  n <- nbrOfFiles(this);
+  n <- length(this);
   verbose && cat(verbose, "Number of files: ", n);
 
 
@@ -59,7 +59,7 @@ setMethodS3("getChecksumObjects", "GenericDataFileSet", function(this, ..., verb
   fullnames <- getFullNames(this);
 
   checksums <- character(length(n));
-  for (ii in seq(length=n)) {
+  for (ii in seq_len(n)) {
     df <- getFile(this, ii);
     checksums[ii] <- getChecksum(df);
     verbose && printf(verbose, "File #%d ('%s') checksum: %s\n", ii, fullnames[ii], checksums[ii]);
